==> packager-2016.1.5-sources.jar: com/xebialabs/deployit/packager/CiNameCache.java <==
/**
 * Copyright © 2014-2016 XebiaLabs Inc. and its affiliates. Use is subject to terms of the enclosed Legal Notice.
 */
package com.xebialabs.deployit.packager;

import com.google.common.base.Joiner;
import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.PropertyKind;
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem;
import com.xebialabs.deployit.plugin.api.xld.Distribution;
import com.xebialabs.deployit.plugin.api.xld.DistributionVersion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.Map;

import static com.google.common.collect.Maps.newHashMap;
import static java.util.EnumSet.of;

class CiNameCache {
    private Map<String,String> idNameCache;

    public CiNameCache() {
        this.idNameCache = newHashMap();
    }

    void scan(DistributionVersion version) {
        Distribution application = version.getDistribution();
        idNameCache.put(application.getId(), application.getName());
        idNameCache.put(version.getId(), version.getName());
        scan(version, null);
    }

    @SuppressWarnings("unchecked")
    private void scan(ConfigurationItem configurationItem, String base) {
        for (PropertyDescriptor propertyDescriptor : configurationItem.getType().getDescriptor().getPropertyDescriptors()) {
            if (of(PropertyKind.LIST_OF_CI, PropertyKind.SET_OF_CI).contains(propertyDescriptor.getKind()) && propertyDescriptor.isAsContainment()) {
                Collection<ConfigurationItem> cis = (Collection<ConfigurationItem>) propertyDescriptor.get(configurationItem);
                if (cis == null) {
                    logger.warn("{}: {}.{} (as-containment, kind {}) should not be null",
                            configurationItem.getId(), configurationItem.getType(), propertyDescriptor.getName(), propertyDescriptor.getKind());
                    continue;
                }
                for (ConfigurationItem ci : cis) {
                    String id = id(base, ci);
                    idNameCache.put(ci.getId(), id);
                    scan(ci, id);
                }
            }
        }
    }

    String lookup(ConfigurationItem ci) {
        if (idNameCache.containsKey(ci.getId())) {
            return idNameCache.get(ci.getId());
        }
        return ci.getId();
    }

    private static String id(String baseId, ConfigurationItem ci) {
        String name = ci.getName();
        return id(baseId, name);
    }

    static String id(String baseId, String name) {
        return Joiner.on("/").skipNulls().join(baseId, name);
    }

    private static final Logger logger = LoggerFactory.getLogger(CiNameCache.class);
}

==> packager-2016.2.2-sources.jar: com/xebialabs/deployit/packager/CiNameCache.java <==
/**
 * Copyright © 2014-2016 XebiaLabs Inc. and its affiliates. Use is subject to terms of the enclosed Legal Notice.
 */
package com.xebialabs.deployit.packager;

import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.PropertyKind;
import com.xebialabs.deployit.plugin.api.udm.Application;
import com.xebialabs.deployit.plugin.api.udm.ConfigurationItem;
import com.xebialabs.deployit.plugin.api.udm.Version;
import com.xebialabs.xlplatform.utils.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import static java.util.EnumSet.of;

class CiNameCache {
    private Map<String,String> idNameCache;

    public CiNameCache() {
        this.idNameCache = new HashMap<>();
    }

    void scan(Version version) {
        Application application = version.getApplication();
        idNameCache.put(application.getId(), application.getName());
        idNameCache.put(version.getId(), version.getName());
        scan(version, null);
    }

    @SuppressWarnings("unchecked")
    private void scan(ConfigurationItem configurationItem, String base) {
        for (PropertyDescriptor propertyDescriptor : configurationItem.getType().getDescriptor().getPropertyDescriptors()) {
            if (of(PropertyKind.LIST_OF_CI, PropertyKind.SET_OF_CI).contains(propertyDescriptor.getKind()) && propertyDescriptor.isAsContainment()) {
                Collection<ConfigurationItem> cis = (Collection<ConfigurationItem>) propertyDescriptor.get(configurationItem);
                if (cis == null) {
                    logger.warn("{}: {}.{} (as-containment, kind {}) should not be null",
                            configurationItem.getId(), configurationItem.getType(), propertyDescriptor.getName(), propertyDescriptor.getKind());
                    continue;
                }
                for (ConfigurationItem ci : cis) {
                    String id = id(base, ci);
                    idNameCache.put(ci.getId(), id);
                    scan(ci, id);
                }
            }
        }
    }

    String lookup(ConfigurationItem ci) {
        if (idNameCache.containsKey(ci.getId())) {
            return idNameCache.get(ci.getId());
        }
        return ci.getId();
    }

    private static String id(String baseId, ConfigurationItem ci) {
        String name = ci.getName();
        return id(baseId, name);
    }

    static String id(String baseId, String name) {
        return Strings.mkString(Arrays.asList(baseId, name), "/");
    }

    private static final Logger logger = LoggerFactory.getLogger(CiNameCache.class);
}
